-- @author		Michigan Gaming and Outdoors
-- Help on the lua by derelky
-- Help to register new fillTypes by Wopster
-- Help on strawblower and lua by The-Alien-Paul and Modelleicher
-- Help on the CornStalk Bales by 4MR Modding
-- Edits and additions by Michigun Gaming & Outdoors
-- @version		1.0


strawMe = {};
strawMe.modDirectory = g_currentModDirectory;
local modDirectory = g_currentModDirectory;
strawMe.modName = g_currentModName;

strawMe.isActive = true

function strawMe.loadFill(hudName, price, massPerLiter, fillTypeName, textName, heapAngle, categoryTable)
	print("***** StrawMe: load fill type "..hudName.." -- "..fillTypeName.." *****")
	local hudFile = "fillPlanes/hud/hud_fill_"..tostring(hudName)..".png"
	local hudFileSmall = "fillPlanes/hud/hud_fill_"..tostring(hudName).."_sml.png"

	local fillType = g_fillTypeManager:addFillType(fillTypeName, g_i18n:getText(textName), true, price, massPerLiter, heapAngle, hudFile, hudFileSmall, modDirectory, nil, {1, 1, 1}, nil, false)

	if fillType == nil then
		print("***** StrawMe: Error adding fillType -- "..tostring(fillTypeName).." *****")       
	else
		local fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fillTypeName)
		
		for _, category in pairs(categoryTable) do
			g_fillTypeManager:addFillTypeToCategory(fillTypeIndex, g_fillTypeManager.nameToCategoryIndex[category])
		end	       
	end
end

function strawMe.loadWindrow(fruitName, windrowNm, windrowLiters)
	print("***** StrawMe: load windrow "..windrowNm.." *****")
	local fruitTable = g_fruitTypeManager:getFruitTypeByName(fruitName)
	local fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(windrowNm)

	fruitTable.windrowName = windrowNm
	fruitTable.hasWindrow = true
	fruitTable.windrowLiterPerSqm = windrowLiters

	table.insert(g_fruitTypeManager.windrowFillTypes, fillTypeIndex, true);

	table.insert(g_fruitTypeManager.fruitTypeIndexToWindrowFillTypeIndex, fruitTable.index, fillTypeIndex);

	table.insert(g_fruitTypeManager.fillTypeIndexToFruitTypeIndex, fillTypeIndex, fruitTable.index);
end

function strawMe.addHeightType(fillType, heightName, heapAngle, colScale, baseOffset, minOffset, maxOffset, fillToGroundScale, allowSmoothing, soundFillType)
	print("***** StrawMe: add heighttype "..heightName.." -- "..fillType.." *****")
	local fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fillType)

	g_fillTypeManager.fillTypeToSample[fillTypeIndex] = g_fillTypeManager.fillTypeToSample[g_fillTypeManager:getFillTypeIndexByName(soundFillType)]

	local diffuseMapFilename = Utils.getFilename("fillPlanes/"..heightName.."_diffuse.png", modDirectory)
	local normalMapFilename = Utils.getFilename("fillPlanes/"..heightName.."_normal.png", modDirectory)	
	local distanceMapFilename = Utils.getFilename("fillPlanes/"..heightName.."_distance_diffuse.png", modDirectory)	

    local ht = g_densityMapHeightManager:addDensityMapHeightType(fillType, math.rad(heapAngle), colScale, baseOffset, minOffset, maxOffset, fillToGroundScale, allowSmoothing, diffuseMapFilename, normalMapFilename, distanceMapFilename, false)
	if ht ~= nil then
		return 1
	else
		print("***** StrawMe: Error adding HeightType -- "..tostring(fillType).." *****")
		return 0
	end
end

local function regFillTypes()
	print("***** StrawMe: register fill types *****")
	strawMe.InitFills(modDirectory, g_fillTypeManger, g_i18n)
end

function strawMe.InitFills(direct, fillTypeMgr, i18n)
	print("***** StrawMe: init fill types *****")
	local priceStraw = g_fillTypeManager:getFillTypeByName("STRAW").pricePerLiter
	local priceSilage = g_fillTypeManager:getFillTypeByName("SILAGE").pricePerLiter
	local strawCategory = {"BULK", "WINDROW", "LOADINGVEHICLE","FORAGEWAGON","SILAGETRAILER","FORK","FORAGEHARVESTER","HAYLOFT"}

	strawMe.loadFill("cornstalks", priceStraw*3, 0.00045, "CORN_STALKS", "fillType_corn_stalks", 35, strawCategory)
	strawMe.loadFill("canolastraw", priceStraw*3, 0.00045, "CANOLASTRAW", "fillType_canola_straw", 35, strawCategory)
	strawMe.loadFill("beanstraw", priceStraw*3, 0.00045, "SOYBEANSTRAW", "fillType_bean_straw", 35, strawCategory)

end

function strawMe.finalizeFills(mission, node)
	print("***** StrawMe: finalize fills *****")

	local y = 0;
	
	--register Fill and Densitymap
	y = y + strawMe.addHeightType("SOYBEANSTRAW", "beanstraw", 35, 0.5, 0.1, 0.1, 1.2, 6, false, "STRAW")
	y = y + strawMe.addHeightType("CORN_STALKS", "cornstraw", 35, 0.5, 0.1, 0.1, 1.2, 6, false, "STRAW")
	y = y + strawMe.addHeightType("CANOLASTRAW", "canolastraw", 35, 0.5, 0.1, 0.1, 1.2, 6, false, "STRAW")

	if(y > 3) then
		print("***** StrawMe: Unable to add necessary HeightTypes *****")
		return
	end

	--strawMe.loadWindrow("CANOLA", "CANOLASTRAW", 3.68)

	--strawMe.loadWindrow("MAIZE", "CORN_STALKS", 4.37)
	
	--strawMe.loadWindrow("SOYBEAN", "SOYBEANSTRAW", 2.15)

	--Load bales -- addBaleType(filename,filltype,isRound,width,height,length,diameter)
	local BeanBaleR = g_baleTypeManager:addBaleType(Utils.getFilename("bales/roundbales/beanBaleRound.i3d", modDirectory), "SOYBEANSTRAW", true, 1.12, 0.9, 2.4, 1.3)
	local CornBaleR = g_baleTypeManager:addBaleType(Utils.getFilename("bales/roundbales/roundbalecorn.i3d", modDirectory), "CORN_STALKS", true, 1.12, 0.9, 2.4, 1.3)
	local CanolaBaleR = g_baleTypeManager:addBaleType(Utils.getFilename("bales/roundbales/roundbaleStraw_w112_d130.i3d", modDirectory), "CANOLASTRAW", true, 1.12, 0.9, 2.4, 1.3)
	local BeanBaleS = g_baleTypeManager:addBaleType(Utils.getFilename("bales/squarebales/beanBaleSquare.i3d", modDirectory), "SOYBEANSTRAW", false, 1.2, 0.9, 2.4, 1.8)
	local CornBaleS = g_baleTypeManager:addBaleType(Utils.getFilename("bales/squarebales/cornstalks.i3d", modDirectory), "CORN_STALKS", false, 1.2, 0.9, 2.4, 1.8)
	local CanolaBaleS = g_baleTypeManager:addBaleType(Utils.getFilename("bales/squarebales/baleStraw240.i3d", modDirectory), "CANOLASTRAW", false, 1.2, 0.9, 2.4, 1.8)
	
	--load MaterialHolder
	local strawMe_fillplane_materialHolderFilename = Utils.getFilename("fillPlanes/corn_bean_straw.i3d", modDirectory)
	strawMe.strawMe_fillplane_materialHolder = loadI3DFile(strawMe_fillplane_materialHolderFilename, false, true, false)
	local strawMe_effect_materialHolderFilename = Utils.getFilename("particleSystems/effect_materialHolder.i3d", modDirectory)
	strawMe.strawMe_effect_materialHolder = loadI3DFile(strawMe_effect_materialHolderFilename, false, true, false)
	local strawMe_particle_materialHolderFilename = Utils.getFilename("particleSystems/particle_materialHolder.i3d", modDirectory)
	strawMe.strawMe_particle_materialHolder = loadI3DFile(strawMe_particle_materialHolderFilename, false, true, false)
	
	-- refresh FilltypeManager
	g_currentMission.hud.fillLevelsDisplay:refreshFillTypes(g_fillTypeManager)		
	
end

function strawMe.onLoadMapFinished(mission, node)
	local cornstalkFilType = g_fillTypeManager:getFillTypeByName("CORN_STALKS")
	local beanstrawFilType = g_fillTypeManager:getFillTypeByName("SOYBEANSTRAW")
	local canstrawFilType = g_fillTypeManager:getFillTypeByName("CANOLASTRAW")
	local maizetable = g_fruitTypeManager:getFruitTypeByName("MAIZE")
	local beantable = g_fruitTypeManager:getFruitTypeByName("SOYBEAN")
	local cantable = g_fruitTypeManager:getFruitTypeByName("CANOLA")
	local maizedex = maizetable.index
	local beandex = beantable.index
	local candex = cantable.index
	maizetable.windrowName = 'CORN_STALKS'
	maizetable.hasWindrow = true
	maizetable.windrowLiterPerSqm = 4.5
	cantable.windrowName = 'CANOLASTRAW'
	cantable.hasWindrow = true
	cantable.windrowLiterPerSqm = 3.75
	beantable.windrowName = 'SOYBEANSTRAW'
	beantable.hasWindrow = true
	beantable.windrowLiterPerSqm = 2.75
	table.insert(g_fruitTypeManager.windrowFillTypes, canstrawFilType.index, true)
	table.insert(g_fruitTypeManager.windrowFillTypes, cornstalkFilType.index, true)
	table.insert(g_fruitTypeManager.windrowFillTypes, beanstrawFilType.index, true)
	
	table.insert(g_fruitTypeManager.fruitTypeIndexToWindrowFillTypeIndex, candex, canstrawFilType.index)
	table.insert(g_fruitTypeManager.fruitTypeIndexToWindrowFillTypeIndex, beandex, beanstrawFilType.index)
	table.insert(g_fruitTypeManager.fruitTypeIndexToWindrowFillTypeIndex, maizedex, cornstalkFilType.index)
	
	table.insert(g_fruitTypeManager.fillTypeIndexToFruitTypeIndex, canstrawFilType.index, candex)
	table.insert(g_fruitTypeManager.fillTypeIndexToFruitTypeIndex, cornstalkFilType.index, maizedex)
	table.insert(g_fruitTypeManager.fillTypeIndexToFruitTypeIndex, beanstrawFilType.index, beandex)

end

function strawMe.addtoseasonseconomy(mission)
		if g_seasons == nil then
		else
			if g_seasons.economy.data.repricing.fillTypes.CORN_STALKS  == nil then
				g_seasons.economy.data.repricing.fillTypes.CORN_STALKS = {}
				for i,j in pairs(g_seasons.economy.data.repricing.fillTypes.STRAW) do
				g_seasons.economy.data.repricing.fillTypes.CORN_STALKS[i]=j
				end
			end
			if g_seasons.economy.data.repricing.fillTypes.SOYBEANSTRAW  == nil then
				g_seasons.economy.data.repricing.fillTypes.SOYBEANSTRAW = {}
				for i,j in pairs(g_seasons.economy.data.repricing.fillTypes.STRAW) do
				g_seasons.economy.data.repricing.fillTypes.SOYBEANSTRAW[i]=j
				end
			end
			if g_seasons.economy.data.repricing.fillTypes.CANOLASTRAW  == nil then
				g_seasons.economy.data.repricing.fillTypes.CANOLASTRAW = {}
				for i,j in pairs(g_seasons.economy.data.repricing.fillTypes.STRAW) do
				g_seasons.economy.data.repricing.fillTypes.CANOLASTRAW[i]=j
				end
			end
		end	
end

function strawMe.load(id, xmlFile, key,customEnvironment)
	-- Add the new Filltype to the SellingStations
	local hasmaize =false
	local hassoybean =false
	local hascornstalks=false
	local hasbeanstraw =false
	local hascanola =false
	local hascanolastraw =false
	local hasStraw =false
	
	local object=id
		
	
	if object == nil then
	else
		local objClass = object.className
		if objClass == "SellingStation" then
			--print("--1--")
			hasmaize =false
			hassoybean =false
			hascornstalks=false
			hasbeanstraw =false
			hasStraw =false
			hascanola =false
			hascanolastraw =false
			for i,filltype in pairs(object.acceptedFillTypes) do
					
				local fillname=g_fillTypeManager.indexToName[i]

				if fillname == "MAIZE" then
					hasmaize=true
								
				end
				if fillname == "SOYBEAN" then
					hassoybean=true
								
				end
				if fillname == "CANOLA" then
					hascanola=true
				end
				if fillname == "CORN_STALKS" then
					hascornstalks=true
								
				end
				if fillname == "SOYBEANSTRAW" then
					hasbeanstraw=true
								
				end
				if fillname == "CANOLASTRAW" then
					hascanolastraw=true
				end
				if fillname == "STRAW" then
					hasStraw = true
				end
				
			end
				
			if hasStraw and not hascornstalks then
				--add cornstalks to the same SellingStation that accept Straw
				local corn_index = g_fillTypeManager:getFillTypeIndexByName("CORN_STALKS")
				local corn_price = g_fillTypeManager:getFillTypeByName("CORN_STALKS").pricePerLiter
				if corn_index ~= nil then
					object:addAcceptedFillType(corn_index,corn_price,true,false)
					object.priceMultipliers[ g_fillTypeManager:getFillTypeIndexByName("CORN_STALKS")]=object.priceMultipliers[ g_fillTypeManager:getFillTypeIndexByName("STRAW")]
				end
			end
			
			if hasStraw and not hasbeanstraw then
				--add beanstraw to the same SellingStation that accept straw
				local bean_index = g_fillTypeManager:getFillTypeIndexByName("SOYBEANSTRAW")
				local bean_price = g_fillTypeManager:getFillTypeByName("SOYBEANSTRAW").pricePerLiter
				if bean_index ~= nil then
					object:addAcceptedFillType(bean_index,bean_price,true,false)
					object.priceMultipliers[ g_fillTypeManager:getFillTypeIndexByName("SOYBEANSTRAW")]=object.priceMultipliers[ g_fillTypeManager:getFillTypeIndexByName("STRAW")]
				end
			end

			if hasStraw and not hascanolastraw then
				--add canolastraw to the same SellingStation that accept straw
				local canola_index = g_fillTypeManager:getFillTypeIndexByName("CANOLASTRAW")
				local canola_price = g_fillTypeManager:getFillTypeByName("CANOLASTRAW").pricePerLiter
				if canola_index ~= nil then
					object:addAcceptedFillType(canola_index,canola_price,true,false)
					object.priceMultipliers[ g_fillTypeManager:getFillTypeIndexByName("CANOLASTRAW")]=object.priceMultipliers[ g_fillTypeManager:getFillTypeIndexByName("STRAW")]
				end
			end
			object:initPricingDynamics()
		else
			--print("--4--")
		end
	end
end

-- Hook here
local function init()
	print("***** StrawMe: Starting registration process *****")
	FillTypeManager.loadMapData = Utils.appendedFunction(FillTypeManager.loadMapData, regFillTypes)

	FSBaseMission.loadMap = Utils.prependedFunction(FSBaseMission.loadMap, strawMe.finalizeFills)

	Mission00.loadItemsFinished = Utils.prependedFunction(Mission00.loadItemsFinished, strawMe.addtoseasonseconomy)

	FSBaseMission.delete = Utils.appendedFunction(FSBaseMission.delete, strawMe.unload)

	FSBaseMission.loadMapFinished = Utils.prependedFunction(FSBaseMission.loadMapFinished, strawMe.onLoadMapFinished)

	SellingStation.load = Utils.appendedFunction(SellingStation.load, strawMe.load)
	
	StrawBlower.onLoad = Utils.overwrittenFunction(StrawBlower.onLoad, strawMe.strawBlowerOnLoadNew)

	HusbandryModuleBase.getIsFillTypeAllowed = Utils.overwrittenFunction(HusbandryModuleBase.getIsFillTypeAllowed, strawMe.getIsFillTypeAllowed)

	HusbandryModuleBase.loadFromXMLFile = Utils.overwrittenFunction(HusbandryModuleBase.loadFromXMLFile, strawMe.loadFromXMLFile)

	strawMe.loadSpecs()
	
	print("***** StrawMe: Registration Complete *****")
end

function strawMe.strawBlowerOnLoadNew( self, superFunc, ...)
	--local returnValue = superFunc(self, ...)
	local returnValue = false;
	if superFunc ~= nil then
		returnValue = superFunc(self, xmlFile);
	end;
	
	local fillUnit = self:getFillUnitByIndex(self.spec_strawBlower.fillUnitIndex)
	
	local names = {"CORN_STALKS", "SOYBEANSTRAW", "CANOLASTRAW"}
	
	for _, name in pairs(names) do
		if g_fillTypeManager:getFillTypeIndexByName(name) ~= nil then
			fillUnit.supportedFillTypes[g_fillTypeManager:getFillTypeIndexByName(name)] = true
		end;
	end;
	
	return returnValue
end;

function strawMe.unload()
	if strawMe.strawMe_particle_materialHolder ~= nil then
		delete(strawMe.strawMe_particle_materialHolder)
		strawMe.strawMe_particle_materialHolder = nil
    end	
    if strawMe.strawMe_fillplane_materialHolder ~= nil then
		delete(strawMe.strawMe_fillplane_materialHolder)
		strawMe.strawMe_fillplane_materialHolder = nil
	end	   
    if strawMe.strawMe_effect_materialHolder ~= nil then
		delete(strawMe.strawMe_effect_materialHolder)
		strawMe.strawMe_effect_materialHolder = nil
	end	   
end

function strawMe.loadSpecs()
    local specName = 'strawMeSpec'

	g_specializationManager:addSpecialization(specName, specName, Utils.getFilename('src/strawMeSpec.lua', g_currentModDirectory), true, nil) 
    for typeName, typeEntry in pairs(g_vehicleTypeManager:getVehicleTypes()) do
        if (SpecializationUtil.hasSpecialization(Dischargeable, typeEntry.specializations)) then
            g_vehicleTypeManager:addSpecialization(typeName, specName)
        end
    end
end

function strawMe.getIsFillTypeAllowed(self, superFunc, fillTypeIndex)
    local strawFillType = g_fillTypeManager:getFillTypeIndexByName("STRAW")
    local corn_stalks = g_fillTypeManager:getFillTypeIndexByName("CORN_STALKS")
    local canola_straw = g_fillTypeManager:getFillTypeIndexByName("CANOLASTRAW")
	local bean_straw = g_fillTypeManager:getFillTypeIndexByName("SOYBEANSTRAW")
	if self.moduleName == "straw" then
		if fillTypeIndex == corn_stalks or fillTypeIndex == canola_straw or fillTypeIndex == bean_straw then
			local origfillTypeIndex = fillTypeIndex
			fillTypeIndex = strawFillType
		end
	end
	for fTypeIndex, state in pairs(self.providedFillTypes) do
		if fTypeIndex == fillTypeIndex and state then
            return true
        end
    end
    return false
end

function strawMe.loadFromXMLFile(self, superFunc, xmlFile, key)
	self.fillCapacity = Utils.getNoNil(getXMLFloat(xmlFile, key.."#fillCapacity"), self.fillCapacity)
	local strawCap = 0.0
	local fillToMove = 0.0
    local i = 0
    while true do
        local fillLevelKey = key .. string.format(".fillLevel(%d)", i)
        if not hasXMLProperty(xmlFile, fillLevelKey) then
            break
        end
        local fillTypeName = getXMLString(xmlFile, fillLevelKey .. "#fillType")
        local fillLevel = getXMLFloat(xmlFile, fillLevelKey .. "#fillLevel")
		if fillTypeName ~= nil and fillLevel ~= nil then
			if fillTypeName == "STRAW" then
				strawCap = fillLevel
			end
			if fillTypeName == "CORN_STALKS" or fillTypeName == "SOYBEANSTRAW" or fillTypeName == "CANOLASTRAW" then
				--skip adding above fill types back in
				fillToMove = fillToMove + fillLevel
			else 
            	local fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fillTypeName)
				if fillTypeIndex ~= nil then
					self:setFillLevel(fillTypeIndex, fillLevel)
					self.providedFillTypes[fillTypeIndex] = true
				end
			end
        end
        i = i + 1
	end
	if fillToMove ~= 0.0 then
		local freeCap = self.fillCapacity - strawCap
		if freeCap > 0.0 then
			if freeCap > fillToMove then
				self:setFillLevel(g_fillTypeManager:getFillTypeIndexByName("STRAW"), strawCap + fillToMove)
			else
				self:setFillLevel(g_fillTypeManager:getFillTypeIndexByName("STRAW"), self.fillCapacity)
			end
		end
	end
end

init()