-- @author		Michigan Gaming and Outdoors
-- @version		1.0

strawMeSpec = {};
  
function strawMeSpec.prerequisitesPresent(specializations)
	return true
end

function strawMeSpec.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "dischargeToObject", strawMeSpec.dischargeToObject)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "getCanDischargeToObject", strawMeSpec.getCanDischargeToObject)
end

function strawMeSpec:dischargeToObject(superFunc, dischargeNode, emptyLiters, object, targetFillUnitIndex)
    local strawFillType = g_fillTypeManager:getFillTypeIndexByName("STRAW")
    local corn_stalks = g_fillTypeManager:getFillTypeIndexByName("CORN_STALKS")
    local canola_straw = g_fillTypeManager:getFillTypeIndexByName("CANOLASTRAW")
    local bean_straw = g_fillTypeManager:getFillTypeIndexByName("SOYBEANSTRAW")
    
    local fillType = self:getDischargeFillType(dischargeNode)

    local origfillType = fillType
    --Allow for New Straw Types to be unloaded at straw module
    if fillType == corn_stalks or fillType == canola_straw or fillType == bean_straw then
        --print("----- I'm a new strawType -----")
        if dischargeNode.dischargeObject.target.moduleName == "straw" then
            --print("----- Changing Fill Type -----")
            fillType = strawFillType
        end
    end

    local supportsFillType = object:getFillUnitSupportsFillType(targetFillUnitIndex, fillType)
    local dischargedLiters = 0
    if supportsFillType then
        local allowFillType = object:getFillUnitAllowsFillType(targetFillUnitIndex, fillType)
        if allowFillType then
            dischargeNode.currentDischargeObject = object
            local delta = object:addFillUnitFillLevel(self:getActiveFarm(), targetFillUnitIndex, emptyLiters, fillType, ToolType.DISCHARGEABLE, dischargeNode.info)
            local unloadInfo = self:getFillVolumeUnloadInfo(dischargeNode.unloadInfoIndex)
            dischargedLiters = self:addFillUnitFillLevel(self:getOwnerFarmId(), dischargeNode.fillUnitIndex, -delta, origfillType, ToolType.UNDEFINED, unloadInfo)
        end
    end
    return dischargedLiters
end

function strawMeSpec:getCanDischargeToObject(superFunc, dischargeNode)
    local strawFillType = g_fillTypeManager:getFillTypeIndexByName("STRAW")
    local corn_stalks = g_fillTypeManager:getFillTypeIndexByName("CORN_STALKS")
    local canola_straw = g_fillTypeManager:getFillTypeIndexByName("CANOLASTRAW")
    local bean_straw = g_fillTypeManager:getFillTypeIndexByName("SOYBEANSTRAW")
    if dischargeNode == nil then
        return false
    end
    local object = dischargeNode.dischargeObject
    if object == nil then
        return false
    end
    local fillType = self:getDischargeFillType(dischargeNode)
    --Allow for New Straw Types to be unloaded at straw module
    if fillType == corn_stalks or fillType == canola_straw or fillType == bean_straw then
        --print("----- I'm a new strawType -----")
        if dischargeNode.dischargeObject.target.moduleName == "straw" then
            --print("----- Changing Fill Type -----")
            fillType = strawFillType
        end
    end

    if not object:getFillUnitSupportsFillType(dischargeNode.dischargeFillUnitIndex, fillType) then
        return false
    end
    local allowFillType = object:getFillUnitAllowsFillType(dischargeNode.dischargeFillUnitIndex, fillType)
    if not allowFillType then
        return false
    end
    if object.getFillUnitFreeCapacity ~= nil and object:getFillUnitFreeCapacity(dischargeNode.dischargeFillUnitIndex, fillType, self:getActiveFarm()) <= 0 then
        return false
    end
    if object.getIsFillAllowedFromFarm ~= nil and not object:getIsFillAllowedFromFarm(self:getActiveFarm()) then
        return false
    end
    -- Adding should only be done if removing is allowed (generally adding is always allowed because it is beneficial to the receiver)
    -- A case where this is not allowed is when this is a pallet where the the controller does not own it (or can access it)
    if self.getMountObject ~= nil then
        local mounter = self:getDynamicMountObject() or self:getMountObject()
        if mounter ~= nil then
            -- if the active farm of the mounter has NO access to farmId fill unit: disallow
            if not g_currentMission.accessHandler:canFarmAccess(mounter:getActiveFarm(), self, true) then
                return false
            end
        end
    end
    return true
end